;------------------------------------------------------------------------------
;  Display
;
;  - handle the NHD-0420 LCD display module
;  - using 8-bit wide data bus 
;
;  7Mar2016 A. Cooper
;  - initial version, from Incline
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
delay	res	1	;a counter
char	res	1	;char storage reg
pos	res	1
loc	res	1
temp	res	1	;temp calc register

;------------------------------------------------------------------------------
;  Available Resources
	global	DispInit, DispClear, DispMove, DispHome, DispWrite
	global	DispHex, DispDec, DispSDec, DispDec2	

;------------------------------------------------------------------------------
;  External Resources
	extern	DelXms, Del1ms, Del10ms, HexToBCD, resHi, resLo

;------------------------------------------------------------------------------
;  Global Definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Module Code
display	code

;------------------------------------------------------------------------------
;  DispInit
;  - initialize the display for use
;  - startup procedure from the Newhaven datasheet
;
DispInit	movlw	0x30	;initialize LCD display
	movwf	dataPort,A
	clrf	dataDir,A	;set direction lines
	bcf	dispCtlDir,dispE,A
	bcf	dispCtlDir,dispRW,A
	bcf	dispCtlDir,dispRS,A
	bcf	dispCtlPort,dispE,A
	bcf	dispCtlPort,dispRW,A
	bcf	dispCtlPort,dispRS,A
	movlw	0x32	;delay 50ms after startup
	call	DelXms
	movlw	0x30	;wake up display
	call	DispCmd
	call	Del10ms	;10ms delay for first wakeup
	movlw	0x30	;second wake up
	call	DispCmd
	call	Del1ms	;1ms delay for second wakeup
	movlw	0x30	;third wake up
	call	DispCmd
	call	Del1ms	;1ms delay for third wakeup
	movlw	0x38	;send setup
	call	DispCmd
	movlw	0x10        
	call	DispCmd
	movlw	0x0C        ;display on, cursor off
	call	DispCmd
	movlw	0x06        ;shift cursor, increment
	call	DispCmd
	return

;------------------------------------------------------------------------------
;  DispClear
;  - clear the display and return to home
;
DispClear	movlw	0x01	;send clear command
	call	DispCmd
	movlw	0x02
	call	DelXms
	return

;------------------------------------------------------------------------------
;  DispHome
;  - home the cursor to upper right pos (0x00)
;
DispHome	movlw	0x02	;send home command
	call	DispCmd
	movlw	0x02
	call	DelXms
	return

;------------------------------------------------------------------------------
;  DispMove
;  - move the cursor to the given position
;  - positions are given in W
;  - first two bits are the row, the last six bits the position in the row
;  - for a 2x16 display:
;      00: row 0, position 0
;      40: row 1, position 0	
;  - for a 4x16 display:
;      00: row 0, position 0
;      40: row 1, position 0
;      80: row 2, position 0
;      C0: row 3, position 0
;      4A: row 1, position 10
;      94: row 3, position 20
;
DispMove	movwf       pos,A       ;save position
            andlw       0x1F
            movwf       loc,A
            rlncf       pos,F,A     ;get top two bits
            rlncf       pos,W,A
            call        DispGetRow  ;get row start address
            addwf       loc,A
            iorlw       0x80        ;make set DDRAM address command
            call        DispCmd
	call	Del1ms	;short delay for command
            return

DispGetRow  andlw       0x03        ;insure valid row 0-3
            movwf       pos,A       ;save position
            movlw       0x00        ;clear W
            btfsc       pos,0,A     ;set bits
             iorlw      0x40
            btfsc       pos,1,A
             iorlw      0x14
            return

;------------------------------------------------------------------------------
;  DispRow
;  - move the cursor to the start of the given row
;  - row given in W
;
DispRow	call        DispGetRow  ;get row address on display
            iorlw       0x80        ;make set DDRAM address comand
            call        DispCmd
            return

;----------------------------------------------------------------------------
;  DispHex
;
;  - send a hexidecimal byte to the LCD
;  - data in W on entry
;
DispHex	movwf	char,A	;temp save char
	swapf	char,W,A	;get upper nibble
	andlw	0x0F
	addlw	0xF6	;check for A-F
	btfsc	STATUS,C,A
	 addlw	0x07
	addlw	0x3A	
	call	DispSend	;send it
	movf	char,W,A	;get lower nibble
	andlw	0x0F
	addlw	0xF6	;check for A-F
	btfsc	STATUS,C,A
	 addlw	0x07
	addlw	0x3A	
	call	DispSend	;send it
	return
	
;----------------------------------------------------------------------------
;  DispDec
;
;  - send a byte to display in decimal
;  - data in W
;
DispDec	call	HexToBCD
	movf	resHi,W,A	;send first digit
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	swapf	resLo,W,A	;send second digit
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	movf	resLo,W,A	;send third digit
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	return

;----------------------------------------------------------------------------
;  DispSDec
;
;  - send a byte to host in decimal
;  - input in signed byte, send negative sign
;  - data in W
;
DispSDec	movwf	temp,A	;temp save value
	movlw	0x20	;space?
	btfsc	temp,7,A
	movlw	0x2D	;or negative sign
	call	DispWrite
	movf	temp,W,A
	btfss	temp,7,A
	 goto	DispDec
	movf	temp,W,A	;inverse number for display
	clrf	temp,A
	subwf	temp,W,A
	goto	DispDec

;----------------------------------------------------------------------------
;  DispDec2
;
;  - send a byte to host in decimal
;  - send only the lower two digits
;  - data in W
;
DispDec2	call	HexToBCD
	swapf	resLo,W,A	;send second digit
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	movf	resLo,W,A	;send third digit
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	return

;------------------------------------------------------------------------------
;  DispWrite
;  - write a character to the display and advance the cursor
;  - charater in ASCII in W on entry
;  - writes a solid block if illegal char sent
;
DispWrite	goto	DispSend
	
	movwf	char,A	;temp save char
	btfsc	char,7,A	;check char for valid
	 goto	DispWrBadCh
	andlw	0x70
	btfsc	STATUS,Z,A
	 goto	DispWrBadCh
	movf	char,w,A
	goto	DispSend
DispWrBadCh	movlw	0xFF
	goto	DispSend

;------------------------------------------------------------------------------
;  DispSend/DispCmd
;  - send one char to the display using control lines
;  - command also raises the command line RS
;  - char in W on entry
;
DispCmd	bcf	dispCtlPort,dispRS,A
	movwf	dataPort,A	;put the char on the data bus
	call	DispDelayS
	bsf	dispCtlPort,dispE,A
	call	DispDelayS
	bcf	dispCtlPort,dispE,A
	call	DispDelayL
	return

DispSend	movwf	dataPort,A	;put the char on the data bus
	bsf	dispCtlPort,dispRS,A
	call	DispDelayS
	bsf	dispCtlPort,dispE,A
	call	DispDelayS
	bcf	dispCtlPort,dispE,A
	call	DispDelayL
	return

DispDelayS	movlw	0x02	;do a short delay using a loop
	goto	DispDelay
DispDelayL	movlw	0x28
DispDelay	movwf	delay,A
DispDelLp	decfsz	delay,f,A
	 goto	DispDelLp
	return
	
	end
;-- end display.asm -----------------------------------------------------------



