;------------------------------------------------------------------------------
;  Serial
;
;  - handle the serial port on the PIC18F66K80
;  - declare a 16 byte buffer
;  - port fixed to 9600,8,N,1 settings, no autobaud
;
;  7Mar2016 A. Cooper
;  - initial version, from Incline
;  6Feb2017 A. Cooper
;  - modified for GenPIC
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"
	
;------------------------------------------------------------------------------
;  Declarations

	UDATA_ACS
serFlag	res	1	;flags for serial port control
serPtr	res	1	;pointers into serial buffers
serBuf	res	16
count	res	1

;bits within serFlag (move to defs.asm for global use)
;serOpen	equ	0	;port is active and ready
;serCmd	equ	1	;command received, <cr> found
;serBufOv	equ	2	;buffer overflow
;serBusy	equ	3	;port is busy, <CTS> not active

;------------------------------------------------------------------------------
;  Available Resources
	global	SerInit, SerGetCh, SerClear	
	global	SerSend, SerSendDec	
	global	SerDump
	global	serFlag, serBuf, serPtr

;------------------------------------------------------------------------------
;  External Resources
	extern	HostHexSp

;------------------------------------------------------------------------------
; hardware assignments
serPort	equ	PORTC	;port for EUSART1
serDir	equ	TRISC
CTS	equ	0x5	;pin for CTS
RTS	equ	0x4	;pin for RTS

;------------------------------------------------------------------------------
;  Module Code
serial	code

;------------------------------------------------------------------------------
;  SerInit
;  - setup primary serial port
;  - setup bits in TXSTA1:
;	7: 0 clock source don't care in asynch
;	6: 0 eight bit data transmission
;	5: 1 transmit enabled
;	4: 0 asynchronous mode
;	3: 0 no break char
;	2: 0 BRGH low speed buad
;	1: 1 TSR empty
;	0: 0 ninth TX bit (if using 9 bit data)
;  - setup bits in RCSTA1:
;	7: 1 serial port enabled
;	6: 0 eight bit data reception
;	5: 0 single recieve (sync only)
;	4: 1 enable receiver
;	3: 0 no address mode
;	2: 0 no framing error
;	1: 0 no overrun error
;	0: 0 ninth RX bit (if using 9 bit data)
;  - setup bits in BAUDCON1:
;	7: 0 autobaud rollover
;	6: 0 RX active (read only)
;	5: 0 RX data not inverted
;	4: 1 TX idle is low
;	3: 0 eight bit baud rate generator
;	2: 0 not used
;	1: 0 no wakeup enable
;	0: 0 autobaud disabled
;  - SPBRG1= 25 for 9600 baud with 0.16% error
;
SerInit	clrf	serFlag,A	;clear flags
	clrf	serPtr,A	;clear internal variables
	movlw	B'00100010'	;set transmission control register
	movwf	TXSTA1,A
	movlw	B'10010000'	;set receive control register
	movwf	RCSTA1,A
	movlw	B'00000000'	;set receive control register
	movwf	BAUDCON1,A
	movlw	0x19	;set baud rate generator for baud rate
	movwf	SPBRG1
	bcf	serDir,CTS	;set CTS and RTS pins to outputs
	bsf	serDir,RTS
	bcf	serPort,CTS	;set CTS and RTS low
	bsf	serPort,RTS
	bsf	PIE1,RC1IE,A	;enable serial receive interrupt
	bsf	serFlag,serOpen,A ;show port open
	return

;------------------------------------------------------------------------------
;  SerGetCh
;  - handle an input charater if present
;  - usually called when a serial interrupt ocurrs
;  - char moved to input buffer
;  - processing flag raised if <cr> received
;
SerGetCh	bcf	serPort,CTS
	btfsc	RCSTA1,OERR	;clear hardware overrun error
	 bcf	RCSTA1,CREN
	bsf	RCSTA1,CREN
	bcf	serFlag,serCmd,A
	movlw	low serBuf	;setup buffer write
	addwf	serPtr,w,A
	movwf	FSR1L,A
	clrf	FSR1H,A
	movf	RCREG1,w,A	;save recieved character
	movwf	INDF1,A
SerGetWait	btfss	PIR1,TX1IF,A ;echo the recieved character
	 goto	SerGetWait
	movwf	TXREG1,A	;send the byte
	incf	serPtr,F,A	;increment pointer
	btfsc	serPtr,4,A	;check for buffer overflow
	 bsf	serFlag,serBufOv,A
	btfsc	serPtr,4,A	;insure no buffer overflow
	 clrf	serPtr,A	;reset on overflow
	;sublw	0x0A	;check for <Newline>
	sublw	0x0D	;check for <CR>
	btfss	STATUS,Z,A
	 return
	movlw	0x0A	;send <NewLine>
SerGetWait2	btfss	PIR1,TX1IF,A
	 goto	SerGetWait2
	movwf	TXREG1	;send the byte
	clrf	serPtr,A	;reset buffer pointer
	bcf	serFlag,serBufOv,A
	bsf	serFlag,serCmd,A
	bsf	serPort,CTS,A ;set CTS while processing command
	bsf	serFlag,serBusy,A
	return

;------------------------------------------------------------------------------
;  SerSend
;  - write a charagter out the primary serial port
;  - char to send in W on entry
;  - non-blocking, checks for finished sending on entry not exit
;  - serial send decimal adds 0x30 to convert a number to ASCII
;
SerSendDec	addlw	0x30	;convert decimal to ASCII
SerSend	btfss	PIR1,TX1IF,A	;check transmission register for empty
	 goto	SerSend
	movwf	TXREG1	;send the byte
	return
	
;------------------------------------------------------------------------------
;  SerClear
;  - clear the buffer and ready for more input
;  - set CTS for input
;  - clear buffer overflow
;
SerClear	bcf	serPort,CTS	;set CTS
	clrf	serPtr,A	;reset buffer pointer
	bcf	serFlag,serCmd,A
	bcf	serFlag,serBufOv,A
	bcf	serFlag,serBusy,A
	return

;------------------------------------------------------------------------------
;  SerDump
;  - dump the contents of the serial buffer in hex
;
SerDump	movff	serPtr,count
	movlw	HIGH serBuf
	movwf	FSR0H,A
	movlw	LOW serBuf
	movwf	FSR0L,A
SerDumpLp	movf	POSTINC0,W,A ;get buffer char
	call	HostHexSp
	decfsz	count,F,A
	 goto	SerDumpLp
	return

	end
;-- end serial.asm ------------------------------------------------------------



