;------------------------------------------------------------------------------
;  state.asm
;  - handle each of the controller states
;  - an init flag in stateFlag will allow each state to initialize
;  - sixteen states numbers are implemented, allowing for eight actual states
;  - a pushbutton is used to advance the state
;  - state 0 is a splash screen state used upon startup only
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2016-2017 all rights reserved
;
;  11Mar2016 A. Cooper
;  - initial version
;  13Jan2017 A. Cooper
;  - modified for GenPIC
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"
	
;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
stateCode	res	1
stateFlag	res	1
temp	res	1
encVal	res	1
testHi	res	1
testLo	res	1
testLED	res	1

;bits in stateFlag
init	equ	0x0

;------------------------------------------------------------------------------
;  Available Resources
	global	StateInit, StateHandle, stateFlag, encVal

;------------------------------------------------------------------------------
;  External Resources
	extern	userFlag
	extern	DispClear, DispMove, DispWrite
	extern	DispHex, DispDec, DispDec2
	extern	MsgMsg, MsgTest
	extern	timeSec, timeMin, timeHour
	extern	ADChan, ADRead
	extern	TempConvert
	extern	resHi, resLo

;------------------------------------------------------------------------------
;  Module Code
state	code

;------------------------------------------------------------------------------
;  StateInit
;
;  - initialize state handler
;
StateInit	clrf	stateCode,A	;set state zero
	clrf	stateFlag,A
	bsf	stateFlag,init,A
	clrf	encVal,A
	clrf	testHi
	clrf	testLo
	clrf	testLED
	goto	State1


;------------------------------------------------------------------------------
;  StateHandle
;
;  - handle the controller states
;
StateHandle	btfsc	userFlag,userPB1,A ;increment state on pushbutton A
	 incf	stateCode,F,A
	btfsc	userFlag,userPB1,A ;set init flag for new state
	 bsf	stateFlag,init,A
	bcf	userFlag,userPB1,A
	movf	stateCode,W,A ;ensure state 0-15
	andlw	0x0F
	movwf	stateCode,A
	movwf	temp,A
	incf	temp,F,A	;offset and x4 for PC words and GOTO
	rlncf	temp,F,A
	rlncf	temp,F,A
	movlw	UPPER StateJmp ;do a computed goto
	movwf	PCLATU,A
	movlw	HIGH StateJmp
	movwf	PCLATH,A
	movf	temp,W,A
	addlw	LOW StateJmp
	btfsc	STATUS,C,A
	 incf	PCLATH,F,A
StateJmp	movwf	PCL,A
	nop
	goto	State0	;00
	goto	State1	;01
	goto	State2	;02
	goto	State3	;03
	goto	State4	;04
	goto	State5	;05
	goto	State6	;06
	goto	State7	;07
	goto	State8	;08
	goto	State9	;09 
	goto	StateA	;0A
	goto	StateB	;0B
	goto	StateC	;0C
	goto	StateD	;0D
	goto	StateE	;0E
	goto	StateF	;0F


;------------------------------------------------------------------------------
;  State0
;
;  - splash screen for startup
;
State0	btfsc	stateFlag,init ;init state?
	 goto	State0Init
	;state handling code here
	return
State0Init	bcf	stateFlag,init,A ;clear flags
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	movlw	0x42
	call	DispMove
	call	MsgTest
	;any other state init code here
	return

;------------------------------------------------------------------------------
;  State1
;
;  - handle state 1
;  - Timer Checks
;
State1	btfsc	stateFlag,init ;init state?
	 goto	State1Init
	btfss	stateFlag,stateUp,A ;update each second
	 return
	bcf	stateFlag,stateUp,A
	movlw	0x44	;update time display
	call	DispMove
	movf	timeHour,W,A
	call	DispDec2
	movlw	':'
	call	DispWrite
	movf	timeMin,W,A
	call	DispDec2
	movlw	':'
	call	DispWrite
	movf	timeSec,W,A
	call	DispDec2
	return
State1Init	bcf	stateFlag,init,A ;clear flags
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	return


;------------------------------------------------------------------------------
;  State2
;
;  - handle state 2
;  - Encoder Test
;
State2	btfsc	stateFlag,init ;init state?
	 goto	State2Init
	;state handling code here
	btfsc	userFlag,userEncUp,A
	 goto	State2Up
	btfsc	userFlag,userEncDn,A
	 goto	State2Down
	btfss	stateFlag,stateUp,A ;update each second
	 return
	bcf	stateFlag,stateUp,A
	goto	State2Disp
State2Up	bcf	userFlag,userEncUp,A
	incf	encVal,F,A
	goto	State2Disp
State2Down	bcf	userFlag,userEncDn,A
	decf	encVal,F,A
State2Disp	movlw	0x44
	call	DispMove
	movf	encVal,W,A
	call	DispHex
	movlw	'h'
	call	DispWrite
	movlw	' '
	call	DispWrite
	movf	encVal,W,A
	call	DispDec
	return
State2Init	bcf	stateFlag,init,A ;clear flags
	bcf	userFlag,userEncUp,A ;ensure no pending encoder tick
	bcf	userFlag,userEncDn,A
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	return


;------------------------------------------------------------------------------
;  State3
;
;  - handle state 3
;  - Analog Input Check
;
State3	btfsc	stateFlag,init ;init state?
	 goto	State3Init
	btfss	stateFlag,stateUp,A ;update each second
	 return
	bcf	stateFlag,stateUp,A
	call	ADRead
	movlw	0x42	;print hex
	call	DispMove
	movf	ADRESH,W,A
	call	DispHex
	movf	ADRESL,W,A
	call	DispHex
	movlw	0x47	;position display cursor for result
	call	DispMove
	movf	ADRESH,W,A	;get result
	movwf	resHi,A
	movf	ADRESL,W,A
	movwf	resLo,A
	call	TempConvert	;convert to decimal
	movlw	'-'	;output sign to display
	btfss	resHi,7,A
	 movlw	' '
	call	DispWrite
	bcf	resHi,7,A
	swapf	resHi,W,A	;output value
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	movf	resHi,W,A
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	swapf	resLo,W,A
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	movlw	'.'
	call	DispWrite
	movf	resLo,W,A
	andlw	0x0F
	addlw	0x30
	call	DispWrite
	return
State3Init	bcf	stateFlag,init,A ;clear flags
	movlw	0x04	;set A/D channel AN4 for thermistor
	call	ADChan
	call	ADRead	;do a dummy read
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	movlw	0x4D	;degree sign and units
	call	DispMove
	movlw	0xDF
	call	DispWrite
	movlw	'C'
	call	DispWrite
	return


;------------------------------------------------------------------------------
;  State4
;
;  - handle state 4
;  - LED Check
;
State4	btfsc	stateFlag,init ;init state?
	 goto	State4Init
	;state handling code here
	btfsc	userFlag,userEncUp,A ;check encoder
	 incf	testLED
	btfsc	userFlag,userEncDn,A
	 decf	testLED
	bcf	userFlag,userEncUp,A
	bcf	userFlag,userEncDn,A
	movlw	0x0F	;mask bit pattern
	andwf	testLED,F,A
	swapf	testLED,W,A	;move to userFlag
	movwf	userFlag,A
	return
State4Init	bcf	stateFlag,init,A ;clear flags
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	;any other state init code here
	return


;------------------------------------------------------------------------------
;  State5
;
;  - handle state 5
;
State5	btfsc	stateFlag,init ;init state?
	 goto	State5Init
	;state handling code here
	return
State5Init	bcf	stateFlag,init,A ;clear flags
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	;any other state init code here
	return


;------------------------------------------------------------------------------
;  State6
;
;  - handle state 6
;
State6	btfsc	stateFlag,init ;init state?
	 goto	State6Init
	;state handling code here
	return
State6Init	bcf	stateFlag,init,A ;clear flags
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	;any other state init code here
	return


;------------------------------------------------------------------------------
;  State7
;
;  - handle state 7
;
State7	btfsc	stateFlag,init ;init state?
	 goto	State7Init
	;state handling code here
	return
State7Init	bcf	stateFlag,init,A ;clear flags
	call	DispClear	;set display for state 1
	movf	stateCode,w,A
	call	MsgMsg
	;any other state init code here
	return


;------------------------------------------------------------------------------
;  State8
;
;  - handle state 8
;
State8	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  State9
;
;  - handle state 9
;
State9	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  StateA
;
;  - handle state A
;
StateA	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  StateB
;
;  - handle state B
;
StateB	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  StateC
;
;  - handle state C
;
StateC	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  StateD
;
;  - handle state D
;
StateD	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  StateE
;
;  - handle state E
;
StateE	incf	stateCode,F,A
	return


;------------------------------------------------------------------------------
;  StateF
;
;  - handle state F
;
StateF	incf	stateCode,F,A
	return

	end
;-- state.asm -----------------------------------------------------------------
