;------------------------------------------------------------------------------
;  User
;
;  - handle the user interface (indicator LED's, encoder, and pushbutton)
;  - the user LED's are handled by testing the flag bits in userFlag, any
;  other code controlling the LED's must simply set the flag bits
;  - the pushbuttons are handled through polling in the PBCheck routine, a
;  debounce timer is used to create stable operation, this timer must be
;  incremented at 100Hz by the interrupt routine
;  - the encoder channels are sampled by the interrupt routine at 1KHz on the
;  TMR2 interrupt used to create the timebase, the following code must be in
;  the ISR to accomplish this...
;	bsf	STATUS,C	;sample encoder values at 1kHz
;	btfss	userRead,userUIO0,A
;	 bcf	STATUS,C
;	rlcf	userEncB,F,A
;	bsf	STATUS,C
;	btfss	userRead,userUIO1,A
;	 bcf	STATUS,C
;	rlcf	userEncA,F,A
;
;  12Jan2017 A. Cooper
;  - initial version
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"
	
;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
userFlag	res	1	;input control flags (global)
userInt	res	1	;input control flags (local)
userPB1Time	res	1	;button debounce timers	
userPB2Time	res	1
userEncA	res	1	;encoder sampling registers
userEncB	res	1
	
;bits in userFlag (move to defs.asm for global use)
;userPB1	equ	0x0	;user pushbutton 1 pressed flag
;userPB2	equ	0x1	;user pushbutton 2 pressed flag
;userEncUp	equ	0x2	;encoder count up flag
;userEncDn	equ	0x3	;encoder count down flag
;userLED1On	equ	0x4	;indicator LED 1 on
;userLED1Fl	equ	0x5	;indicator LED 1 flash
;userLED2On	equ	0x6	;indicator LED 2 on
;userLED2Fl	equ	0x7	;indicator LED 2 flash
	
;bits in userInt
pb1Down	equ	0x0
pb2Down	equ	0x1

;------------------------------------------------------------------------------
;  Available Resources
	global	UserInit, UserLEDCheck, UserPBCheck, UserEncCheck
	global	userEncA, userEncB
	global	userFlag, userPB1Time, userPB2Time
	global	UserLED1On, UserLED1Off, UserLED2On, UserLED2Off

;------------------------------------------------------------------------------
;  External Resources
	extern	timeCent

;------------------------------------------------------------------------------
;  Module Code
user	code

;------------------------------------------------------------------------------
;  UserInit
;  - initialize the user interface
;  - set the hardware IO directions
;  - turn the indicator LED's off
;
UserInit	movlw	b'00001111'	;set user port directions
	movwf	userDir,A
	bcf	userPort,userLED1,A
	bcf	userPort,userLED1,A
	clrf	userInt	;clear all flags
	clrf	userFlag
	return

;------------------------------------------------------------------------------
;  UserLED
;  - turn the LED's on and off
;
UserLED1On	bsf	userFlag,userLED1On
	return
UserLED1Off	bcf	userFlag,userLED1On
	return
UserLED2On	bsf	userFlag,userLED2On
	return
UserLED2Off	bcf	userFlag,userLED2On
	return

;------------------------------------------------------------------------------
;  UserLEDCheck
;  - control the indicator LED's
;
UserLEDCheck	btfsc	userFlag,userLED1Fl,A ;flash?
	 goto	UserLCk1Flsh
	btfss	userFlag,userLED1On,A ;set LED1
	 bcf	userPort,userLED1,A
	btfsc	userFlag,userLED1On,A
	 bsf	userPort,userLED1,A
	goto	UserLCk2
UserLCk1Flsh	btfss	timeCent,4,A	;flash LED1
	 bcf	userPort,userLED1,A
	btfsc	timeCent,4,A
	 bsf	userPort,userLED1,A
UserLCk2	btfsc	userFlag,userLED2Fl,A ;flash?
	 goto	UserLCk2Flsh
	btfss	userFlag,userLED2On,A ;set LED2
	 bcf	userPort,userLED2,A
	btfsc	userFlag,userLED2On,A
	 bsf	userPort,userLED2,A
	return
UserLCk2Flsh	btfss	timeCent,4,A	;flash LED2
	 bsf	userPort,userLED2,A
	btfsc	timeCent,4,A
	 bcf	userPort,userLED2,A
	return

;------------------------------------------------------------------------------
;  UserPBCheck
;  - check and debounce the pushbuttons
;  - buttons are at userPort bits 2 and 3
;  - rotary encoder pushbutton is shared with PB1, use only one
;  - results indicated by setting flags in userFlag bits userPB1 and userPB2
;  - a timer is used to debounce the button, must be pressed for 0.1s minimum
;  - button press registered upon release of pushbutton
;
UserPBCheck	btfss	userRead,userUIO2 ;check PB1
	 goto	UserPBCk1D
	btfsc	userInt,pb1Down,A ;push in progress?
	 goto	UserPBCk1P
	clrf	userPB1Time,A ;no push, keep timer clear
	goto	UserPBCk2
UserPBCk1P	bcf	userInt,pb1Down,A ;clear push in progress flag
	movf	userPB1Time,W,A ;check debounce timer
	andlw	0xF8
	btfss	STATUS,Z
	 bsf	userFlag,userPB1,A ;button pressed!
	goto	UserPBCk2
UserPBCk1D	bsf	userInt,pb1Down,A ;show push in progress
	btfsc	userPB1Time,7,A ;ensure no timer overflow
	 decf	userPB1Time,F,A
UserPBCk2	btfss	userRead,userUIO3 ;check PB2
	 goto	UserPBCk2D
	btfsc	userInt,pb2Down,A ;push in progress?
	 goto	UserPBCk2P
	clrf	userPB2Time,A ;no push, keep timer clear
	return
UserPBCk2P	bcf	userInt,pb2Down,A ;clear push in progress flag
	movf	userPB2Time,W,A ;check debounce timer
	andlw	0xF8
	btfss	STATUS,Z
	 bsf	userFlag,userPB2,A ;button pressed!
	return
UserPBCk2D	bsf	userInt,pb2Down,A ;show push in progress
	btfsc	userPB2Time,7,A ;ensure no timer overflow
	 decf	userPB2Time,F,A
	return

;------------------------------------------------------------------------------
;  UserEncCheck
;  - check and debounce the rotary encoder
;  - encoder A and B are at userPort bits 1 and 0
;  - sampled at 1KHz to the userEncA and userEncB registers
;  - Bit pattern in the sample registers used to detect rotation as shown
;	A: 111xx000 and B: xxx11xxx -> CW rotation
;	A: 111xx000 and B: xxx00xxx -> CCW rotation
;  - results indicated by setting flags in userFlag bits userEncUp and userEncDn
;
UserEncCheck	movf	userEncA,W,A ;check A sample register for pattern
	andlw	0xE7	;mask bits
	sublw	0xE0	;check pattern
	btfss	STATUS,Z
	 return
	clrf	userEncA,A	;ensure no double count
	movf	userEncB,W,A ;check B level at edge
	andlw	0x18	;mask center bits
	btfsc	STATUS,Z	;check for CCW
	 bsf	userFlag,userEncDn,A
	sublw	0x18	;check for CW
	btfsc	STATUS,Z
	 bsf	userFlag,userEncUp,A
	return
	
	end
;-- end user.asm --------------------------------------------------------------
